@@ -1,4 +1,4 @@
-/* $Header: /cvsroot/osrs/libtiff/libtiff/tif_getimage.c,v 1.27 2003/10/03 11:21:23 dron Exp $ */
+/* $Id: tif_getimage.c,v 1.43 2004/12/19 17:11:17 dron Exp $ */
 
 /*
  * Copyright (c) 1991-1997 Sam Leffler
@@ -30,7 +30,6 @@
  * Read and return a packed RGBA image.
  */
 #include "tiffiop.h"
-#include <assert.h>
 #include <stdio.h>
 
 static	int gtTileContig(TIFFRGBAImage*, uint32*, uint32, uint32);
@@ -42,6 +41,28 @@ static	int pickTileSeparateCase(TIFFRGBAImage*);
 
 static	const char photoTag[] = "PhotometricInterpretation";
 
+/* 
+ * Helper constants used in Orientation tag handling
+ */
+#define FLIP_VERTICALLY 0x01
+#define FLIP_HORIZONTALLY 0x02
+
+/*
+ * Color conversion constants. We will define display types here.
+ */
+
+TIFFDisplay display_sRGB = {
+	{			/* XYZ -> luminance matrix */
+		{  3.2410F, -1.5374F, -0.4986F },
+		{  -0.9692F, 1.8760F, 0.0416F },
+		{  0.0556F, -0.2040F, 1.0570F }
+	},	
+	100.0F, 100.0F, 100.0F,	/* Light o/p for reference white */
+	255, 255, 255,		/* Pixel values for ref. white */
+	1.0F, 1.0F, 1.0F,	/* Residual light o/p for black pixel */
+	2.4F, 2.4F, 2.4F,	/* Gamma values for the three guns */
+};
+
 /*
  * Check the image to see if TIFFReadRGBAImage can deal with it.
  * 1/0 is returned according to whether or not the image can
@@ -117,7 +138,6 @@ TIFFRGBAImageOK(TIFF* tif, char emsg[1024])
 	    return (0);
 	}
 	break;
-#ifdef CMYK_SUPPORT
     case PHOTOMETRIC_SEPARATED:
 	if (td->td_inkset != INKSET_CMYK) {
 	    sprintf(emsg, "Sorry, can not handle separated image with %s=%d",
@@ -130,7 +150,6 @@ TIFFRGBAImageOK(TIFF* tif, char emsg[1024])
 	    return (0);
 	}
 	break;
-#endif
     case PHOTOMETRIC_LOGL:
 	if (td->td_compression != COMPRESSION_SGILOG) {
 	    sprintf(emsg, "Sorry, LogL data must have %s=%d",
@@ -151,6 +170,8 @@ TIFFRGBAImageOK(TIFF* tif, char emsg[1024])
 	    return (0);
 	}
 	break;
+    case PHOTOMETRIC_CIELAB:
+	break;
     default:
 	sprintf(emsg, "Sorry, can not handle image with %s=%d",
 	    photoTag, photometric);
@@ -162,20 +183,22 @@ TIFFRGBAImageOK(TIFF* tif, char emsg[1024])
 void
 TIFFRGBAImageEnd(TIFFRGBAImage* img)
 {
-    if (img->Map)
-	_TIFFfree(img->Map), img->Map = NULL;
-    if (img->BWmap)
-	_TIFFfree(img->BWmap), img->BWmap = NULL;
-    if (img->PALmap)
-	_TIFFfree(img->PALmap), img->PALmap = NULL;
-    if (img->ycbcr)
-	_TIFFfree(img->ycbcr), img->ycbcr = NULL;
+	if (img->Map)
+		_TIFFfree(img->Map), img->Map = NULL;
+	if (img->BWmap)
+		_TIFFfree(img->BWmap), img->BWmap = NULL;
+	if (img->PALmap)
+		_TIFFfree(img->PALmap), img->PALmap = NULL;
+	if (img->ycbcr)
+		_TIFFfree(img->ycbcr), img->ycbcr = NULL;
+	if (img->cielab)
+		_TIFFfree(img->cielab), img->cielab = NULL;
 
-    if( img->redcmap ) {
-        _TIFFfree( img->redcmap );
-        _TIFFfree( img->greencmap );
-        _TIFFfree( img->bluecmap );
-    }
+	if( img->redcmap ) {
+		_TIFFfree( img->redcmap );
+		_TIFFfree( img->greencmap );
+		_TIFFfree( img->bluecmap );
+	}
 }
 
 static int
@@ -206,6 +229,7 @@ TIFFRGBAImageBegin(TIFFRGBAImage* img, TIFF* tif, int stop, char emsg[1024])
     img->redcmap = NULL;
     img->greencmap = NULL;
     img->bluecmap = NULL;
+    img->req_orientation = ORIENTATION_BOTLEFT;	    /* It is the default */
     
     img->tif = tif;
     img->stoponerr = stop;
@@ -215,7 +239,7 @@ TIFFRGBAImageBegin(TIFFRGBAImage* img, TIFF* tif, int stop, char emsg[1024])
     case 8: case 16:
 	break;
     default:
-	sprintf(emsg, "Sorry, can not image with %d-bit samples",
+	sprintf(emsg, "Sorry, can not handle images with %d-bit samples",
 	    img->bitspersample);
 	return (0);
     }
@@ -223,11 +247,11 @@ TIFFRGBAImageBegin(TIFFRGBAImage* img, TIFF* tif, int stop, char emsg[1024])
     TIFFGetFieldDefaulted(tif, TIFFTAG_SAMPLESPERPIXEL, &img->samplesperpixel);
     TIFFGetFieldDefaulted(tif, TIFFTAG_EXTRASAMPLES,
 	&extrasamples, &sampleinfo);
-    if (extrasamples == 1)
+    if (extrasamples > 1)
     {
 	switch (sampleinfo[0]) {
 	case EXTRASAMPLE_UNSPECIFIED:	/* Workaround for some images without */
-		if (img->samplesperpixel == 4)	/* correct info about alpha channel */
+		if (img->samplesperpixel > 3)	/* correct info about alpha channel */
 			img->alpha = EXTRASAMPLE_ASSOCALPHA;
 		break;
 	case EXTRASAMPLE_ASSOCALPHA:	/* data is pre-multiplied */
@@ -237,7 +261,7 @@ TIFFRGBAImageBegin(TIFFRGBAImage* img, TIFF* tif, int stop, char emsg[1024])
 	}
     }
 
-#if DEFAULT_EXTRASAMPLE_AS_ALPHA == 1
+#ifdef DEFAULT_EXTRASAMPLE_AS_ALPHA
     if( !TIFFGetField(tif, TIFFTAG_PHOTOMETRIC, &img->photometric))
         img->photometric = PHOTOMETRIC_MINISWHITE;
 
@@ -287,9 +311,9 @@ TIFFRGBAImageBegin(TIFFRGBAImage* img, TIFF* tif, int stop, char emsg[1024])
 	    return (0);
         }
 
-        memcpy( img->redcmap, red_orig, n_color * 2 );
-        memcpy( img->greencmap, green_orig, n_color * 2 );
-        memcpy( img->bluecmap, blue_orig, n_color * 2 );
+        _TIFFmemcpy( img->redcmap, red_orig, n_color * 2 );
+        _TIFFmemcpy( img->greencmap, green_orig, n_color * 2 );
+        _TIFFmemcpy( img->bluecmap, blue_orig, n_color * 2 );
         
 	/* fall thru... */
     case PHOTOMETRIC_MINISWHITE:
@@ -375,6 +399,8 @@ TIFFRGBAImageBegin(TIFFRGBAImage* img, TIFF* tif, int stop, char emsg[1024])
 	img->photometric = PHOTOMETRIC_RGB;		/* little white lie */
 	img->bitspersample = 8;
 	break;
+    case PHOTOMETRIC_CIELAB:
+	break;
     default:
 	sprintf(emsg, "Sorry, can not handle image with %s=%d",
 	    photoTag, img->photometric);
@@ -384,6 +410,7 @@ TIFFRGBAImageBegin(TIFFRGBAImage* img, TIFF* tif, int stop, char emsg[1024])
     img->BWmap = NULL;
     img->PALmap = NULL;
     img->ycbcr = NULL;
+    img->cielab = NULL;
     TIFFGetField(tif, TIFFTAG_IMAGEWIDTH, &img->width);
     TIFFGetField(tif, TIFFTAG_IMAGELENGTH, &img->height);
     TIFFGetFieldDefaulted(tif, TIFFTAG_ORIENTATION, &img->orientation);
@@ -414,29 +441,9 @@ TIFFRGBAImageGet(TIFFRGBAImage* img, uint32* raster, uint32 w, uint32 h)
 }
 
 /*
- * Read the specified image into an ABGR-format raster.
+ * Read the specified image into an ABGR-format rastertaking in account
+ * specified orientation.
  */
-int
-TIFFReadRGBAImage(TIFF* tif,
-		  uint32 rwidth, uint32 rheight, uint32* raster, int stop)
-{
-    char emsg[1024];
-    TIFFRGBAImage img;
-    int ok;
-
-    if (TIFFRGBAImageOK(tif, emsg) &&
-	TIFFRGBAImageBegin(&img, tif, stop, emsg)) {
-	/* XXX verify rwidth and rheight against width and height */
-	ok = TIFFRGBAImageGet(&img, raster+(rheight-img.height)*rwidth,
-	    rwidth, img.height);
-	TIFFRGBAImageEnd(&img);
-    } else {
-	TIFFError(TIFFFileName(tif), emsg);
-	ok = 0;
-    }
-    return (ok);
-}
-
 int
 TIFFReadRGBAImageOriented(TIFF* tif,
 			  uint32 rwidth, uint32 rheight, uint32* raster,
@@ -448,85 +455,11 @@ TIFFReadRGBAImageOriented(TIFF* tif,
 
     if (TIFFRGBAImageOK(tif, emsg) &&
 	TIFFRGBAImageBegin(&img, tif, stop, emsg)) {
+	img.req_orientation = orientation;
 	/* XXX verify rwidth and rheight against width and height */
 	ok = TIFFRGBAImageGet(&img, raster+(rheight-img.height)*rwidth,
 	    rwidth, img.height);
 	TIFFRGBAImageEnd(&img);
-
-	/* 
-	 * TIFFRGBAImageGet() always returns bottom-left oriented raster.
-	 * Wi will rotate it accordingly with user request.
-	 */
-	if (orientation == ORIENTATION_TOPLEFT ||
-	    orientation == ORIENTATION_LEFTTOP) {
-		uint32 *linebuf = (uint32*)_TIFFmalloc(rwidth * sizeof(uint32));
-		uint32 *top, *bottom;
-
-		for (top = raster, bottom = raster + (rheight - 1) * rwidth;
-		     top < bottom;
-		     top+=rwidth, bottom-=rwidth) {
-			_TIFFmemcpy(linebuf, top, rwidth * sizeof(uint32));
-			_TIFFmemcpy(top, bottom, rwidth * sizeof(uint32));
-			_TIFFmemcpy(bottom, linebuf, rwidth * sizeof(uint32));
-		}
-
-		_TIFFfree(linebuf);
-	} else if (orientation == ORIENTATION_TOPRIGHT ||
-		   orientation == ORIENTATION_RIGHTTOP) {
-		uint32 *linebuf = (uint32*)_TIFFmalloc(rwidth * sizeof(uint32));
-		uint32 *top, *bottom;
-
-		for (top = raster, bottom = raster + (rheight - 1) * rwidth;
-		/* XXX: Use `<=' because we should revert center line too */
-		     top <= bottom;
-		     top+=rwidth, bottom-=rwidth) {
-			uint32 *left = top;
-			uint32 *right = left + rwidth - 1;
-			
-			while ( left < right ) {
-				uint32 temp = *left;
-				*left = *right;
-				*right = temp;
-				left++, right--;
-			}
-
-			/* 
-			 * XXX: check for center line, otherwise
-			 * it will be reverted back
-			 */
-			if (top != bottom) {
-				left = bottom;
-				right = left + rwidth - 1;
-				while ( left < right ) {
-					uint32 temp = *left;
-					*left = *right;
-					*right = temp;
-					left++, right--;
-				}
-			}
-
-			_TIFFmemcpy(linebuf, top, rwidth * sizeof(uint32));
-			_TIFFmemcpy(top, bottom, rwidth * sizeof(uint32));
-			_TIFFmemcpy(bottom, linebuf, rwidth * sizeof(uint32));
-		}
-
-		_TIFFfree(linebuf);
-	} else if (orientation == ORIENTATION_BOTRIGHT ||
-		   orientation == ORIENTATION_RIGHTBOT) {
-		uint32 line;
-
-		for (line = 0; line < rheight; line++) {
-			uint32 *left = raster + (line * rwidth);
-			uint32 *right = left + rwidth - 1;
-
-			while ( left < right ) {
-				uint32 temp = *left;
-				*left = *right;
-				*right = temp;
-				left++, right--;
-			}
-		}
-	}
     } else {
 	TIFFError(TIFFFileName(tif), emsg);
 	ok = 0;
@@ -534,34 +467,77 @@ TIFFReadRGBAImageOriented(TIFF* tif,
     return (ok);
 }
 
-static uint32
-setorientation(TIFFRGBAImage* img, uint32 h)
+/*
+ * Read the specified image into an ABGR-format raster. Use bottom left
+ * origin for raster by default.
+ */
+int
+TIFFReadRGBAImage(TIFF* tif,
+		  uint32 rwidth, uint32 rheight, uint32* raster, int stop)
 {
-    TIFF* tif = img->tif;
-    uint32 y;
+	return TIFFReadRGBAImageOriented(tif, rwidth, rheight, raster,
+					 ORIENTATION_BOTLEFT, stop);
+}
 
-    switch (img->orientation) {
-    case ORIENTATION_BOTRIGHT:
-    case ORIENTATION_RIGHTBOT:	/* XXX */
-    case ORIENTATION_LEFTBOT:	/* XXX */
-	TIFFWarning(TIFFFileName(tif), "using bottom-left orientation");
-	img->orientation = ORIENTATION_BOTLEFT;
-	/* fall thru... */
-    case ORIENTATION_BOTLEFT:
-	y = 0;
-	break;
-    case ORIENTATION_TOPRIGHT:
-    case ORIENTATION_RIGHTTOP:	/* XXX */
-    case ORIENTATION_LEFTTOP:	/* XXX */
-    default:
-	TIFFWarning(TIFFFileName(tif), "using top-left orientation");
-	img->orientation = ORIENTATION_TOPLEFT;
-	/* fall thru... */
-    case ORIENTATION_TOPLEFT:
-	y = h-1;
-	break;
-    }
-    return (y);
+static int 
+setorientation(TIFFRGBAImage* img)
+{
+	switch (img->orientation) {
+		case ORIENTATION_TOPLEFT:
+		case ORIENTATION_LEFTTOP:
+			if (img->req_orientation == ORIENTATION_TOPRIGHT ||
+			    img->req_orientation == ORIENTATION_RIGHTTOP)
+				return FLIP_HORIZONTALLY;
+			else if (img->req_orientation == ORIENTATION_BOTRIGHT ||
+			    img->req_orientation == ORIENTATION_RIGHTBOT)
+				return FLIP_HORIZONTALLY | FLIP_VERTICALLY;
+			else if (img->req_orientation == ORIENTATION_BOTLEFT ||
+			    img->req_orientation == ORIENTATION_LEFTBOT)
+				return FLIP_VERTICALLY;
+			else
+				return 0;
+		case ORIENTATION_TOPRIGHT:
+		case ORIENTATION_RIGHTTOP:
+			if (img->req_orientation == ORIENTATION_TOPLEFT ||
+			    img->req_orientation == ORIENTATION_LEFTTOP)
+				return FLIP_HORIZONTALLY;
+			else if (img->req_orientation == ORIENTATION_BOTRIGHT ||
+			    img->req_orientation == ORIENTATION_RIGHTBOT)
+				return FLIP_VERTICALLY;
+			else if (img->req_orientation == ORIENTATION_BOTLEFT ||
+			    img->req_orientation == ORIENTATION_LEFTBOT)
+				return FLIP_HORIZONTALLY | FLIP_VERTICALLY;
+			else
+				return 0;
+		case ORIENTATION_BOTRIGHT:
+		case ORIENTATION_RIGHTBOT:
+			if (img->req_orientation == ORIENTATION_TOPLEFT ||
+			    img->req_orientation == ORIENTATION_LEFTTOP)
+				return FLIP_HORIZONTALLY | FLIP_VERTICALLY;
+			else if (img->req_orientation == ORIENTATION_TOPRIGHT ||
+			    img->req_orientation == ORIENTATION_RIGHTTOP)
+				return FLIP_VERTICALLY;
+			else if (img->req_orientation == ORIENTATION_BOTLEFT ||
+			    img->req_orientation == ORIENTATION_LEFTBOT)
+				return FLIP_HORIZONTALLY;
+			else
+				return 0;
+		case ORIENTATION_BOTLEFT:
+		case ORIENTATION_LEFTBOT:
+			if (img->req_orientation == ORIENTATION_TOPLEFT ||
+			    img->req_orientation == ORIENTATION_LEFTTOP)
+				return FLIP_VERTICALLY;
+			else if (img->req_orientation == ORIENTATION_TOPRIGHT ||
+			    img->req_orientation == ORIENTATION_RIGHTTOP)
+				return FLIP_HORIZONTALLY | FLIP_VERTICALLY;
+			else if (img->req_orientation == ORIENTATION_BOTRIGHT ||
+			    img->req_orientation == ORIENTATION_RIGHTBOT)
+				return FLIP_HORIZONTALLY;
+			else
+				return 0;
+		default:	/* NOTREACHED */
+			return 0;
+	}
 }
 
 /*
@@ -575,24 +551,33 @@ gtTileContig(TIFFRGBAImage* img, uint32* raster, uint32 w, uint32 h)
 {
     TIFF* tif = img->tif;
     tileContigRoutine put = img->put.contig;
-    uint16 orientation;
-    uint32 col, row, y, rowstoread, ret = 1;
+    uint32 col, row, y, rowstoread;
     uint32 pos;
     uint32 tw, th;
-    u_char* buf;
+    unsigned char* buf;
     int32 fromskew, toskew;
     uint32 nrow;
+    int ret = 1, flip;
 
-    buf = (u_char*) _TIFFmalloc(TIFFTileSize(tif));
+    buf = (unsigned char*) _TIFFmalloc(TIFFTileSize(tif));
     if (buf == 0) {
 	TIFFError(TIFFFileName(tif), "No space for tile buffer");
 	return (0);
     }
+    _TIFFmemset(buf, 0, TIFFTileSize(tif));
     TIFFGetField(tif, TIFFTAG_TILEWIDTH, &tw);
     TIFFGetField(tif, TIFFTAG_TILELENGTH, &th);
-    y = setorientation(img, h);
-    orientation = img->orientation;
-    toskew = -(int32) (orientation == ORIENTATION_TOPLEFT ? tw+w : tw-w);
+
+    flip = setorientation(img);
+    if (flip & FLIP_VERTICALLY) {
+	    y = h - 1;
+	    toskew = -(int32)(tw + w);
+    }
+    else {
+	    y = 0;
+	    toskew = -(int32)(tw - w);
+    }
+     
     for (row = 0; row < h; row += nrow)
     {
         rowstoread = th - (row + img->row_offset) % th;
@@ -623,13 +608,28 @@ gtTileContig(TIFFRGBAImage* img, uint32* raster, uint32 w, uint32 h)
             {
                 (*put)(img, raster+y*w+col, col, y, tw, nrow, 0, toskew, buf + pos);
             }
-
-	    
         }
 
-        y += (orientation == ORIENTATION_TOPLEFT ? -(int32) nrow : (int32) nrow);
+        y += (flip & FLIP_VERTICALLY ? -(int32) nrow : (int32) nrow);
     }
     _TIFFfree(buf);
+
+    if (flip & FLIP_HORIZONTALLY) {
+	    uint32 line;
+
+	    for (line = 0; line < h; line++) {
+		    uint32 *left = raster + (line * w);
+		    uint32 *right = left + w - 1;
+		    
+		    while ( left < right ) {
+			    uint32 temp = *left;
+			    *left = *right;
+			    *right = temp;
+			    left++, right--;
+		    }
+	    }
+    }
+
     return (ret);
 }
 
@@ -644,38 +644,46 @@ gtTileSeparate(TIFFRGBAImage* img, uint32* raster, uint32 w, uint32 h)
 {
     TIFF* tif = img->tif;
     tileSeparateRoutine put = img->put.separate;
-    uint16 orientation;
     uint32 col, row, y, rowstoread;
     uint32 pos;
     uint32 tw, th;
-    u_char* buf;
-    u_char* r;
-    u_char* g;
-    u_char* b;
-    u_char* a;
+    unsigned char* buf;
+    unsigned char* r;
+    unsigned char* g;
+    unsigned char* b;
+    unsigned char* a;
     tsize_t tilesize;
     int32 fromskew, toskew;
     int alpha = img->alpha;
     uint32 nrow;
-    int ret = 1;
+    int ret = 1, flip;
 
     tilesize = TIFFTileSize(tif);
-    buf = (u_char*) _TIFFmalloc(4*tilesize);
+    buf = (unsigned char*) _TIFFmalloc(4*tilesize);
     if (buf == 0) {
 	TIFFError(TIFFFileName(tif), "No space for tile buffer");
 	return (0);
     }
+    _TIFFmemset(buf, 0, 4*tilesize);
     r = buf;
     g = r + tilesize;
     b = g + tilesize;
     a = b + tilesize;
     if (!alpha)
-	memset(a, 0xff, tilesize);
+	_TIFFmemset(a, 0xff, tilesize);
     TIFFGetField(tif, TIFFTAG_TILEWIDTH, &tw);
     TIFFGetField(tif, TIFFTAG_TILELENGTH, &th);
-    y = setorientation(img, h);
-    orientation = img->orientation;
-    toskew = -(int32) (orientation == ORIENTATION_TOPLEFT ? tw+w : tw-w);
+
+    flip = setorientation(img);
+    if (flip & FLIP_VERTICALLY) {
+	    y = h - 1;
+	    toskew = -(int32)(tw + w);
+    }
+    else {
+	    y = 0;
+	    toskew = -(int32)(tw - w);
+    }
+
     for (row = 0; row < h; row += nrow) 
     {
         rowstoread = th - (row + img->row_offset) % th;
@@ -720,16 +728,31 @@ gtTileSeparate(TIFFRGBAImage* img, uint32* raster, uint32 w, uint32 h)
                 (*put)(img, raster+y*w+col, col, y,
                        npix, nrow, fromskew, toskew + fromskew, 
                        r + pos, g + pos, b + pos, a + pos);
-            } 
-            else 
-            {
+            } else {
                 (*put)(img, raster+y*w+col, col, y,
                        tw, nrow, 0, toskew, r + pos, g + pos, b + pos, a + pos);
             }
         }
 
-        y += (orientation == ORIENTATION_TOPLEFT ?-(int32) nrow : (int32) nrow);
+        y += (flip & FLIP_VERTICALLY ?-(int32) nrow : (int32) nrow);
     }
+
+    if (flip & FLIP_HORIZONTALLY) {
+	    uint32 line;
+
+	    for (line = 0; line < h; line++) {
+		    uint32 *left = raster + (line * w);
+		    uint32 *right = left + w - 1;
+		    
+		    while ( left < right ) {
+			    uint32 temp = *left;
+			    *left = *right;
+			    *right = temp;
+			    left++, right--;
+		    }
+	    }
+    }
+
     _TIFFfree(buf);
     return (ret);
 }
@@ -745,24 +768,31 @@ gtStripContig(TIFFRGBAImage* img, uint32* raster, uint32 w, uint32 h)
 {
     TIFF* tif = img->tif;
     tileContigRoutine put = img->put.contig;
-    uint16 orientation;
     uint32 row, y, nrow, rowstoread;
     uint32 pos;
-    u_char* buf;
+    unsigned char* buf;
     uint32 rowsperstrip;
     uint32 imagewidth = img->width;
     tsize_t scanline;
     int32 fromskew, toskew;
-    int ret = 1;
+    int ret = 1, flip;
 
-    buf = (u_char*) _TIFFmalloc(TIFFStripSize(tif));
+    buf = (unsigned char*) _TIFFmalloc(TIFFStripSize(tif));
     if (buf == 0) {
 	TIFFError(TIFFFileName(tif), "No space for strip buffer");
 	return (0);
     }
-    y = setorientation(img, h);
-    orientation = img->orientation;
-    toskew = -(int32) (orientation == ORIENTATION_TOPLEFT ? w+w : w-w);
+    _TIFFmemset(buf, 0, TIFFStripSize(tif));
+
+    flip = setorientation(img);
+    if (flip & FLIP_VERTICALLY) {
+	    y = h - 1;
+	    toskew = -(int32)(w + w);
+    } else {
+	    y = 0;
+	    toskew = -(int32)(w - w);
+    }
+
     TIFFGetFieldDefaulted(tif, TIFFTAG_ROWSPERSTRIP, &rowsperstrip);
     scanline = TIFFScanlineSize(tif);
     fromskew = (w < imagewidth ? imagewidth - w : 0);
@@ -782,8 +812,25 @@ gtStripContig(TIFFRGBAImage* img, uint32* raster, uint32 w, uint32 h)
 
         pos = ((row + img->row_offset) % rowsperstrip) * scanline;
         (*put)(img, raster+y*w, 0, y, w, nrow, fromskew, toskew, buf + pos);
-        y += (orientation == ORIENTATION_TOPLEFT ?-(int32) nrow : (int32) nrow);
+        y += (flip & FLIP_VERTICALLY ? -(int32) nrow : (int32) nrow);
     }
+
+    if (flip & FLIP_HORIZONTALLY) {
+	    uint32 line;
+
+	    for (line = 0; line < h; line++) {
+		    uint32 *left = raster + (line * w);
+		    uint32 *right = left + w - 1;
+		    
+		    while ( left < right ) {
+			    uint32 temp = *left;
+			    *left = *right;
+			    *right = temp;
+			    left++, right--;
+		    }
+	    }
+    }
+
     _TIFFfree(buf);
     return (ret);
 }
@@ -799,9 +846,8 @@ gtStripSeparate(TIFFRGBAImage* img, uint32* raster, uint32 w, uint32 h)
 {
     TIFF* tif = img->tif;
     tileSeparateRoutine put = img->put.separate;
-    uint16 orientation;
-    u_char *buf;
-    u_char *r, *g, *b, *a;
+    unsigned char *buf;
+    unsigned char *r, *g, *b, *a;
     uint32 row, y, nrow, rowstoread;
     uint32 pos;
     tsize_t scanline;
@@ -810,22 +856,31 @@ gtStripSeparate(TIFFRGBAImage* img, uint32* raster, uint32 w, uint32 h)
     tsize_t stripsize;
     int32 fromskew, toskew;
     int alpha = img->alpha;
-    int	ret = 1;
+    int	ret = 1, flip;
 
     stripsize = TIFFStripSize(tif);
-    r = buf = (u_char *)_TIFFmalloc(4*stripsize);
+    r = buf = (unsigned char *)_TIFFmalloc(4*stripsize);
     if (buf == 0) {
 	TIFFError(TIFFFileName(tif), "No space for tile buffer");
 	return (0);
     }
+    _TIFFmemset(buf, 0, 4*stripsize);
     g = r + stripsize;
     b = g + stripsize;
     a = b + stripsize;
     if (!alpha)
-	memset(a, 0xff, stripsize);
-    y = setorientation(img, h);
-    orientation = img->orientation;
-    toskew = -(int32) (orientation == ORIENTATION_TOPLEFT ? w+w : w-w);
+	_TIFFmemset(a, 0xff, stripsize);
+
+    flip = setorientation(img);
+    if (flip & FLIP_VERTICALLY) {
+	    y = h - 1;
+	    toskew = -(int32)(w + w);
+    }
+    else {
+	    y = 0;
+	    toskew = -(int32)(w - w);
+    }
+
     TIFFGetFieldDefaulted(tif, TIFFTAG_ROWSPERSTRIP, &rowsperstrip);
     scanline = TIFFScanlineSize(tif);
     fromskew = (w < imagewidth ? imagewidth - w : 0);
@@ -867,8 +922,25 @@ gtStripSeparate(TIFFRGBAImage* img, uint32* raster, uint32 w, uint32 h)
         pos = ((row + img->row_offset) % rowsperstrip) * scanline;
         (*put)(img, raster+y*w, 0, y, w, nrow, fromskew, toskew, r + pos, g + pos, 
                b + pos, a + pos);
-        y += (orientation == ORIENTATION_TOPLEFT ? -(int32) nrow : (int32) nrow);
+        y += (flip & FLIP_VERTICALLY ? -(int32) nrow : (int32) nrow);
     }
+
+    if (flip & FLIP_HORIZONTALLY) {
+	    uint32 line;
+
+	    for (line = 0; line < h; line++) {
+		    uint32 *left = raster + (line * w);
+		    uint32 *right = left + w - 1;
+		    
+		    while ( left < right ) {
+			    uint32 temp = *left;
+			    *left = *right;
+			    *right = temp;
+			    left++, right--;
+		    }
+	    }
+    }
+
     _TIFFfree(buf);
     return (ret);
 }
@@ -932,7 +1004,7 @@ gtStripSeparate(TIFFRGBAImage* img, uint32* raster, uint32 w, uint32 h)
 #define	SKEW(r,g,b,skew)	{ r += skew; g += skew; b += skew; }
 #define	SKEW4(r,g,b,a,skew)	{ r += skew; g += skew; b += skew; a+= skew; }
 
-#define A1 ((uint32)(0xffL<<24))
+#define A1 (((uint32)0xffL)<<24)
 #define	PACK(r,g,b)	\
 	((uint32)(r)|((uint32)(g)<<8)|((uint32)(b)<<16)|A1)
 #define	PACK4(r,g,b,a)	\
@@ -950,7 +1022,7 @@ static void name(\
     uint32 x, uint32 y, \
     uint32 w, uint32 h, \
     int32 fromskew, int32 toskew, \
-    u_char* pp \
+    unsigned char* pp \
 )
 
 /*
@@ -1336,7 +1408,7 @@ static void name(\
     uint32 x, uint32 y, \
     uint32 w, uint32 h,\
     int32 fromskew, int32 toskew,\
-    u_char* r, u_char* g, u_char* b, u_char* a\
+    unsigned char* r, unsigned char* g, unsigned char* b, unsigned char* a\
 )
 
 /*
@@ -1472,25 +1544,39 @@ DECLARESepPutFunc(putRGBUAseparate16bittile)
 }
 
 /*
- * YCbCr -> RGB conversion and packing routines.  The colorspace
- * conversion algorithm comes from the IJG v5a code; see below
- * for more information on how it works.
+ * 8-bit packed CIE L*a*b 1976 samples => RGB
+ */
+DECLAREContigPutFunc(putcontig8bitCIELab)
+{
+	float X, Y, Z;
+	uint32 r, g, b;
+	(void) y;
+	fromskew *= 3;
+	while (h-- > 0) {
+		for (x = w; x-- > 0;) {
+			TIFFCIELabToXYZ(img->cielab,
+					(unsigned char)pp[0],
+					(signed char)pp[1],
+					(signed char)pp[2],
+					&X, &Y, &Z);
+			TIFFXYZToRGB(img->cielab, X, Y, Z, &r, &g, &b);
+			*cp++ = PACK(r, g, b);
+			pp += 3;
+		}
+		cp += toskew;
+		pp += fromskew;
+	}
+}
+
+/*
+ * YCbCr -> RGB conversion and packing routines.
  */
 
-#define	YCbCrtoRGB(dst, yc) {						\
-    int Y = (yc);							\
-    dst = PACK(								\
-	clamptab[Y+Crrtab[Cr]],						\
-	clamptab[Y + (int)((Cbgtab[Cb]+Crgtab[Cr])>>16)],		\
-	clamptab[Y+Cbbtab[Cb]]);					\
+#define	YCbCrtoRGB(dst, Y) {						\
+	uint32 r, g, b;							\
+	TIFFYCbCrtoRGB(img->ycbcr, (Y), Cb, Cr, &r, &g, &b);		\
+	dst = PACK(r, g, b);						\
 }
-#define	YCbCrSetup							\
-    TIFFYCbCrToRGB* ycbcr = img->ycbcr;					\
-    int* Crrtab = ycbcr->Cr_r_tab;					\
-    int* Cbbtab = ycbcr->Cb_b_tab;					\
-    int32* Crgtab = ycbcr->Cr_g_tab;					\
-    int32* Cbgtab = ycbcr->Cb_g_tab;					\
-    TIFFRGBValue* clamptab = ycbcr->clamptab
 
 /*
  * 8-bit packed YCbCr samples => RGB 
@@ -1507,18 +1593,16 @@ static void putcontig8bitYCbCrGenericTile(
     uint32 x, uint32 y, 
     uint32 w, uint32 h, 
     int32 fromskew, int32 toskew, 
-    u_char* pp,
+    unsigned char* pp,
     int h_group, 
     int v_group )
 
 {
-    YCbCrSetup;
-    
     uint32* cp1 = cp+w+toskew;
     uint32* cp2 = cp1+w+toskew;
     uint32* cp3 = cp2+w+toskew;
     int32 incr = 3*w+4*toskew;
-    int     Cb, Cr;
+    int32   Cb, Cr;
     int     group_size = v_group * h_group + 2;
 
     (void) y;
@@ -1526,7 +1610,7 @@ static void putcontig8bitYCbCrGenericTile(
 
     for( yy = 0; yy < h; yy++ )
     {
-        u_char *pp_line;
+        unsigned char *pp_line;
         int     y_line_group = yy / v_group;
         int     y_remainder = yy - y_line_group * v_group;
 
@@ -1575,7 +1659,6 @@ static void putcontig8bitYCbCrGenericTile(
  */
 DECLAREContigPutFunc(putcontig8bitYCbCr44tile)
 {
-    YCbCrSetup;
     uint32* cp1 = cp+w+toskew;
     uint32* cp2 = cp1+w+toskew;
     uint32* cp3 = cp2+w+toskew;
@@ -1588,8 +1671,8 @@ DECLAREContigPutFunc(putcontig8bitYCbCr44tile)
         for (; h >= 4; h -= 4) {
             x = w>>2;
             do {
-                int Cb = pp[16];
-                int Cr = pp[17];
+                int32 Cb = pp[16];
+                int32 Cr = pp[17];
 
                 YCbCrtoRGB(cp [0], pp[ 0]);
                 YCbCrtoRGB(cp [1], pp[ 1]);
@@ -1617,8 +1700,8 @@ DECLAREContigPutFunc(putcontig8bitYCbCr44tile)
     } else {
         while (h > 0) {
             for (x = w; x > 0;) {
-                int Cb = pp[16];
-                int Cr = pp[17];
+                int32 Cb = pp[16];
+                int32 Cr = pp[17];
                 switch (x) {
                 default:
                     switch (h) {
@@ -1673,7 +1756,6 @@ DECLAREContigPutFunc(putcontig8bitYCbCr44tile)
  */
 DECLAREContigPutFunc(putcontig8bitYCbCr42tile)
 {
-    YCbCrSetup;
     uint32* cp1 = cp+w+toskew;
     int32 incr = 2*toskew+w;
 
@@ -1683,8 +1765,8 @@ DECLAREContigPutFunc(putcontig8bitYCbCr42tile)
         for (; h >= 2; h -= 2) {
             x = w>>2;
             do {
-                int Cb = pp[8];
-                int Cr = pp[9];
+                int32 Cb = pp[8];
+                int32 Cr = pp[9];
                 
                 YCbCrtoRGB(cp [0], pp[0]);
                 YCbCrtoRGB(cp [1], pp[1]);
@@ -1704,8 +1786,8 @@ DECLAREContigPutFunc(putcontig8bitYCbCr42tile)
     } else {
         while (h > 0) {
             for (x = w; x > 0;) {
-                int Cb = pp[8];
-                int Cr = pp[9];
+                int32 Cb = pp[8];
+                int32 Cr = pp[9];
                 switch (x) {
                 default:
                     switch (h) {
@@ -1752,15 +1834,13 @@ DECLAREContigPutFunc(putcontig8bitYCbCr42tile)
  */
 DECLAREContigPutFunc(putcontig8bitYCbCr41tile)
 {
-    YCbCrSetup;
-
     (void) y;
     /* XXX adjust fromskew */
     do {
 	x = w>>2;
 	do {
-	    int Cb = pp[4];
-	    int Cr = pp[5];
+	    int32 Cb = pp[4];
+	    int32 Cr = pp[5];
 
 	    YCbCrtoRGB(cp [0], pp[0]);
 	    YCbCrtoRGB(cp [1], pp[1]);
@@ -1773,8 +1853,8 @@ DECLAREContigPutFunc(putcontig8bitYCbCr41tile)
 
         if( (w&3) != 0 )
         {
-	    int Cb = pp[4];
-	    int Cr = pp[5];
+	    int32 Cb = pp[4];
+	    int32 Cr = pp[5];
 
             switch( (w&3) ) {
               case 3: YCbCrtoRGB(cp [2], pp[2]);
@@ -1798,7 +1878,6 @@ DECLAREContigPutFunc(putcontig8bitYCbCr41tile)
  */
 DECLAREContigPutFunc(putcontig8bitYCbCr22tile)
 {
-    YCbCrSetup;
     uint32* cp1 = cp+w+toskew;
     int32 incr = 2*toskew+w;
 
@@ -1808,8 +1887,8 @@ DECLAREContigPutFunc(putcontig8bitYCbCr22tile)
         for (; h >= 2; h -= 2) {
             x = w>>1;
             do {
-                int Cb = pp[4];
-                int Cr = pp[5];
+                int32 Cb = pp[4];
+                int32 Cr = pp[5];
 
                 YCbCrtoRGB(cp [0], pp[0]);
                 YCbCrtoRGB(cp [1], pp[1]);
@@ -1825,8 +1904,8 @@ DECLAREContigPutFunc(putcontig8bitYCbCr22tile)
     } else {
         while (h > 0) {
             for (x = w; x > 0;) {
-                int Cb = pp[4];
-                int Cr = pp[5];
+                int32 Cb = pp[4];
+                int32 Cr = pp[5];
                 switch (x) {
                 default:
                     switch (h) {
@@ -1863,15 +1942,13 @@ DECLAREContigPutFunc(putcontig8bitYCbCr22tile)
  */
 DECLAREContigPutFunc(putcontig8bitYCbCr21tile)
 {
-    YCbCrSetup;
-
     (void) y;
     fromskew = (fromskew * 4) / 2;
     do {
 	x = w>>1;
 	do {
-	    int Cb = pp[2];
-	    int Cr = pp[3];
+	    int32 Cb = pp[2];
+	    int32 Cr = pp[3];
 
 	    YCbCrtoRGB(cp[0], pp[0]); 
 	    YCbCrtoRGB(cp[1], pp[1]);
@@ -1882,8 +1959,8 @@ DECLAREContigPutFunc(putcontig8bitYCbCr21tile)
 
         if( (w&1) != 0 )
         {
-	    int Cb = pp[2];
-	    int Cr = pp[3];
+	    int32 Cb = pp[2];
+	    int32 Cr = pp[3];
             
             YCbCrtoRGB(cp [0], pp[0]);
 
@@ -1901,15 +1978,13 @@ DECLAREContigPutFunc(putcontig8bitYCbCr21tile)
  */
 DECLAREContigPutFunc(putcontig8bitYCbCr11tile)
 {
-    YCbCrSetup;
-
     (void) y;
     fromskew *= 3;
     do {
         x = w; /* was x = w>>1; patched 2000/09/25 warmerda@home.com */ 
 	do {
-	    int Cb = pp[1];
-	    int Cr = pp[2];
+	    int32 Cb = pp[1];
+	    int32 Cr = pp[2];
 
 	    YCbCrtoRGB(*cp++, pp[0]);
 
@@ -1919,125 +1994,87 @@ DECLAREContigPutFunc(putcontig8bitYCbCr11tile)
 	pp += fromskew;
     } while (--h);
 }
-#undef	YCbCrSetup
 #undef	YCbCrtoRGB
 
-#define	LumaRed			coeffs[0]
-#define	LumaGreen		coeffs[1]
-#define	LumaBlue		coeffs[2]
-#define	SHIFT			16
-#define	FIX(x)			((int32)((x) * (1L<<SHIFT) + 0.5))
-#define	ONE_HALF		((int32)(1<<(SHIFT-1)))
-
-/*
- * Initialize the YCbCr->RGB conversion tables.  The conversion
- * is done according to the 6.0 spec:
- *
- *    R = Y + Cr*(2 - 2*LumaRed)
- *    B = Y + Cb*(2 - 2*LumaBlue)
- *    G =   Y
- *        - LumaBlue*Cb*(2-2*LumaBlue)/LumaGreen
- *        - LumaRed*Cr*(2-2*LumaRed)/LumaGreen
- *
- * To avoid floating point arithmetic the fractional constants that
- * come out of the equations are represented as fixed point values
- * in the range 0...2^16.  We also eliminate multiplications by
- * pre-calculating possible values indexed by Cb and Cr (this code
- * assumes conversion is being done for 8-bit samples).
- */
-static void
-TIFFYCbCrToRGBInit(TIFFYCbCrToRGB* ycbcr, TIFF* tif)
-{
-    TIFFRGBValue* clamptab;
-    float* coeffs;
-    int i;
-
-    clamptab = (TIFFRGBValue*)(
-	(tidata_t) ycbcr+TIFFroundup(sizeof (TIFFYCbCrToRGB), sizeof (long)));
-    _TIFFmemset(clamptab, 0, 256);		/* v < 0 => 0 */
-    ycbcr->clamptab = (clamptab += 256);
-    for (i = 0; i < 256; i++)
-	clamptab[i] = (TIFFRGBValue) i;
-    _TIFFmemset(clamptab+256, 255, 2*256);	/* v > 255 => 255 */
-    TIFFGetFieldDefaulted(tif, TIFFTAG_YCBCRCOEFFICIENTS, &coeffs);
-    _TIFFmemcpy(ycbcr->coeffs, coeffs, 3*sizeof (float));
-    { float f1 = 2-2*LumaRed;		int32 D1 = FIX(f1);
-      float f2 = LumaRed*f1/LumaGreen;	int32 D2 = -FIX(f2);
-      float f3 = 2-2*LumaBlue;		int32 D3 = FIX(f3);
-      float f4 = LumaBlue*f3/LumaGreen;	int32 D4 = -FIX(f4);
-      int x;
-
-      ycbcr->Cr_r_tab = (int*) (clamptab + 3*256);
-      ycbcr->Cb_b_tab = ycbcr->Cr_r_tab + 256;
-      ycbcr->Cr_g_tab = (int32*) (ycbcr->Cb_b_tab + 256);
-      ycbcr->Cb_g_tab = ycbcr->Cr_g_tab + 256;
-      /*
-       * i is the actual input pixel value in the range 0..255
-       * Cb and Cr values are in the range -128..127 (actually
-       * they are in a range defined by the ReferenceBlackWhite
-       * tag) so there is some range shifting to do here when
-       * constructing tables indexed by the raw pixel data.
-       *
-       * XXX handle ReferenceBlackWhite correctly to calculate
-       *     Cb/Cr values to use in constructing the tables.
-       */
-      for (i = 0, x = -128; i < 256; i++, x++) {
-	  ycbcr->Cr_r_tab[i] = (int)((D1*x + ONE_HALF)>>SHIFT);
-	  ycbcr->Cb_b_tab[i] = (int)((D3*x + ONE_HALF)>>SHIFT);
-	  ycbcr->Cr_g_tab[i] = D2*x;
-	  ycbcr->Cb_g_tab[i] = D4*x + ONE_HALF;
-      }
-    }
-}
-#undef	SHIFT
-#undef	ONE_HALF
-#undef	FIX
-#undef	LumaBlue
-#undef	LumaGreen
-#undef	LumaRed
-
 static tileContigRoutine
 initYCbCrConversion(TIFFRGBAImage* img)
 {
-    uint16 hs, vs;
+	static char module[] = "initCIELabConversion";
+
+	float *luma, *refBlackWhite;
+	uint16 hs, vs;
 
-    if (img->ycbcr == NULL) {
-	img->ycbcr = (TIFFYCbCrToRGB*) _TIFFmalloc(
-	      TIFFroundup(sizeof (TIFFYCbCrToRGB), sizeof (long))
-	    + 4*256*sizeof (TIFFRGBValue)
-	    + 2*256*sizeof (int)
-	    + 2*256*sizeof (int32)
-	);
 	if (img->ycbcr == NULL) {
-	    TIFFError(TIFFFileName(img->tif),
-		"No space for YCbCr->RGB conversion state");
-	    return (NULL);
+	    img->ycbcr = (TIFFYCbCrToRGB*) _TIFFmalloc(
+		    TIFFroundup(sizeof (TIFFYCbCrToRGB), sizeof (long))
+		    + 4*256*sizeof (TIFFRGBValue)
+		    + 2*256*sizeof (int)
+		    + 3*256*sizeof (int32)
+	    );
+	    if (img->ycbcr == NULL) {
+		    TIFFError(module,
+			      "No space for YCbCr->RGB conversion state");
+		    return (NULL);
+	    }
 	}
-	TIFFYCbCrToRGBInit(img->ycbcr, img->tif);
-    } else {
-	float* coeffs;
 
-	TIFFGetFieldDefaulted(img->tif, TIFFTAG_YCBCRCOEFFICIENTS, &coeffs);
-	if (_TIFFmemcmp(coeffs, img->ycbcr->coeffs, 3*sizeof (float)) != 0)
-	    TIFFYCbCrToRGBInit(img->ycbcr, img->tif);
-    }
-    /*
-     * The 6.0 spec says that subsampling must be
-     * one of 1, 2, or 4, and that vertical subsampling
-     * must always be <= horizontal subsampling; so
-     * there are only a few possibilities and we just
-     * enumerate the cases.
-     */
-    TIFFGetFieldDefaulted(img->tif, TIFFTAG_YCBCRSUBSAMPLING, &hs, &vs);
-    switch ((hs<<4)|vs) {
-    case 0x44: return (putcontig8bitYCbCr44tile);
-    case 0x42: return (putcontig8bitYCbCr42tile);
-    case 0x41: return (putcontig8bitYCbCr41tile);
-    case 0x22: return (putcontig8bitYCbCr22tile);
-    case 0x21: return (putcontig8bitYCbCr21tile);
-    case 0x11: return (putcontig8bitYCbCr11tile);
-    }
-    return (NULL);
+	TIFFGetFieldDefaulted(img->tif, TIFFTAG_YCBCRCOEFFICIENTS, &luma);
+	TIFFGetFieldDefaulted(img->tif, TIFFTAG_REFERENCEBLACKWHITE,
+			      &refBlackWhite);
+	if (TIFFYCbCrToRGBInit(img->ycbcr, luma, refBlackWhite) < 0)
+		return NULL;
+
+	/*
+	 * The 6.0 spec says that subsampling must be
+	 * one of 1, 2, or 4, and that vertical subsampling
+	 * must always be <= horizontal subsampling; so
+	 * there are only a few possibilities and we just
+	 * enumerate the cases.
+	 */
+	TIFFGetFieldDefaulted(img->tif, TIFFTAG_YCBCRSUBSAMPLING, &hs, &vs);
+	switch ((hs<<4)|vs) {
+		case 0x44: return (putcontig8bitYCbCr44tile);
+		case 0x42: return (putcontig8bitYCbCr42tile);
+		case 0x41: return (putcontig8bitYCbCr41tile);
+		case 0x22: return (putcontig8bitYCbCr22tile);
+		case 0x21: return (putcontig8bitYCbCr21tile);
+		case 0x11: return (putcontig8bitYCbCr11tile);
+	}
+
+	return (NULL);
+}
+
+static tileContigRoutine
+initCIELabConversion(TIFFRGBAImage* img)
+{
+	static char module[] = "initCIELabConversion";
+
+	float   *whitePoint;
+	float   refWhite[3];
+
+	if (!img->cielab) {
+		img->cielab = (TIFFCIELabToRGB *)
+			_TIFFmalloc(sizeof(TIFFCIELabToRGB));
+		if (!img->cielab) {
+			TIFFError(module,
+			    "No space for CIE L*a*b*->RGB conversion state.");
+			return NULL;
+		}
+	}
+
+	TIFFGetFieldDefaulted(img->tif, TIFFTAG_WHITEPOINT, &whitePoint);
+	refWhite[1] = 100.0F;
+	refWhite[0] = whitePoint[0] / whitePoint[1] * refWhite[1];
+	refWhite[2] = (1.0F - whitePoint[0] - whitePoint[1])
+		      / whitePoint[1] * refWhite[1];
+	if (TIFFCIELabToRGBInit(img->cielab, &display_sRGB, refWhite) < 0) {
+		TIFFError(module,
+		    "Failed to initialize CIE L*a*b*->RGB conversion state.");
+		_TIFFfree(img->cielab);
+		return NULL;
+	}
+
+	return putcontig8bitCIELab;
 }
 
 /*
@@ -2338,6 +2375,10 @@ pickTileContigCase(TIFFRGBAImage* img)
 	    if (img->bitspersample == 8)
 		put = initYCbCrConversion(img);
 	    break;
+	case PHOTOMETRIC_CIELAB:
+	    if (img->bitspersample == 8)
+		put = initCIELabConversion(img);
+	    break;
 	}
     }
     return ((img->put.contig = put) != 0);
@@ -2481,9 +2522,10 @@ TIFFReadRGBATile(TIFF* tif, uint32 col, uint32 row, uint32 * raster)
      * Setup the RGBA reader.
      */
     
-    if (!TIFFRGBAImageOK(tif, emsg) || !TIFFRGBAImageBegin(&img, tif, 0, emsg)) {
-	TIFFError(TIFFFileName(tif), emsg);
-        return( 0 );
+    if (!TIFFRGBAImageOK(tif, emsg) 
+	|| !TIFFRGBAImageBegin(&img, tif, 0, emsg)) {
+	    TIFFError(TIFFFileName(tif), emsg);
+	    return( 0 );
     }
 
     /*
@@ -2525,8 +2567,7 @@ TIFFReadRGBATile(TIFF* tif, uint32 col, uint32 row, uint32 * raster)
     if( read_xsize == tile_xsize && read_ysize == tile_ysize )
         return( ok );
 
-    for( i_row = 0; i_row < read_ysize; i_row++ )
-    {
+    for( i_row = 0; i_row < read_ysize; i_row++ ) {
         memmove( raster + (tile_ysize - i_row - 1) * tile_xsize,
                  raster + (read_ysize - i_row - 1) * read_xsize,
                  read_xsize * sizeof(uint32) );
@@ -2534,11 +2575,12 @@ TIFFReadRGBATile(TIFF* tif, uint32 col, uint32 row, uint32 * raster)
                      0, sizeof(uint32) * (tile_xsize - read_xsize) );
     }
 
-    for( i_row = read_ysize; i_row < tile_ysize; i_row++ )
-    {
+    for( i_row = read_ysize; i_row < tile_ysize; i_row++ ) {
         _TIFFmemset( raster + (tile_ysize - i_row - 1) * tile_xsize,
                      0, sizeof(uint32) * tile_xsize );
     }
 
     return (ok);
 }
+
+/* vim: set ts=8 sts=8 sw=8 noet: */
